
/* This program is derived from the C program described by William J. Thompson.
   William J. Thompson, Angular Momentum: an illustrated guide 
                       to rotational symmetries for physical systems,
                       John Wiley and Sons Inc., 1994.
*/

#define _USE_MATH_DEFINES 
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "am.h"


double Djmpm(int tj, int tmp, int tm, double beta)
  
 {
   double norm, cb2, sb2, sum, term;
   int minmax[3], txmin, txmax,tx;
   
   norm = sqrt(Fctrl((tj+tmp)/2)*Fctrl((tj-tmp)/2)*
         Fctrl((tj+tm)/2)*Fctrl((tj-tm)/2));
   cb2 = cos(beta/2);
   sb2 = sin(beta/2);
   minmax[1] = 0;
   minmax[2] = tmp -tm;
   txmin = Max(minmax,2);
   minmax[1] = tj + tmp;
   minmax[2] = tj - tm;
   txmax = Min(minmax, 2);
   sum = 0;
   for ( tx = txmin; tx <= txmax; tx = tx+2)
     {
       term = Power(cb2,tj-tx+(tmp-tm)/2)*
              Power(sb2,tx+(tm-tmp)/2)/
              (Fctrl((tj+tmp-tx)/2)*
               Fctrl((tj-tm-tx)/2)*Fctrl(tx/2)*
               Fctrl((tx+tm-tmp)/2));
       if (tx !=4*(tx/4))
         {
           term = -term;
         };
       sum = sum + norm*term;
     };
     return sum;
 }

        
int main(void)
 {
   double beta,dj,j,mp,m;
   int tj, tmp, tm;
   double Djmpm();
      
   printf("Angular Momentum\n");
   printf("Reduced Rotation Matrix Elements\n");

   tj = 0;
   while (tj >= 0)
     {
       printf("\n\nInput j (-1 to end): ");
       scanf("%lf",&j);
       tj = Twice(j);
       if (tj < 0 )
         {
           printf("\nEnd rotation matrix elements\n");
           exit(0);
         };
       printf("\nInput m', m, beta(in degrees): ");
       scanf("%lf,%lf,%lf",&mp,&m,&beta);
       tmp = Twice(mp);
       tm = Twice(m);
	   beta=beta*(M_PI/180); 
       if (abs(tmp) > tj || abs(tm) > tj)
         {
          printf("\n!!m' or m is too big; try again");
         }
       else
         {
          dj = Djmpm(tj,tmp,tm,beta);
          printf("\n djm'm(beta) = %lf", dj);
         };
     };
  return 0;     
   
 }
