/*  
     The W3j procedure was derived from the C program described by William J. Thompson.
          William J. Thompson, Angular Momentum: An Illustrated Guide 
               to Rotational Symmetries for Physical Systems,
                    John Wiley and Sons Inc., 1994

     This program adds two angular momemta j1, and j2
*/

#define _USE_MATH_DEFINES 
#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <math.h>
#include "am.h"

typedef struct fraction
 {
  int numer;  /* numerator */
  int denom;  /* denominator */
  char s;     /* sign of the fraction */
 } frac;


 /* W3j() determines the Wigner 3j coefficient using Racah's formula */

double W3j(double a, double b,
           double c, double ma,
           double mb, double mc)
           
  {
    double n1, n2, n3, d1, d2, d3;
    double norm, sum, phase, term;
    int minmax[3], tkmin, tkmax, tk;
    
  /*  Normalization Factorials  */
  
     n1 = Fctrl((c+a-b))*Fctrl((c-a+b));
     n2 = Fctrl((a+b-c));
     n3 = Fctrl((c-mc))*Fctrl((c+mc));
     d1 = Fctrl((a+b+c)+1);
     d2 = Fctrl((a-ma))*Fctrl((a+ma));
     d3 = Fctrl((b-mb))*Fctrl((b+mb));
 	 norm = Power(-1.0,(a-b-mc))*sqrt(n1*n2*n3/(d1*d2*d3));
     
     minmax[1] = 0;
     minmax[2] = b-a-mc; 
     tkmin = Max(minmax,2); 
     minmax[1] = c-a+b;  
     minmax[2] = c-mc;   
     tkmax = Min(minmax,2);
     sum = 0;
     phase = Power(-1.0, (tkmin+b+mb));
      
     for(tk= tkmin; tk <= tkmax; tk++)
       {
         n1 = Fctrl((b+c+ma-tk));
         n2 = Fctrl((a-ma+tk));
         d1 = Fctrl(tk)*Fctrl((c-a+b-tk));  
         d2 = Fctrl((c-mc-tk));
         d3 = Fctrl((tk+a-b+mc));
         term = phase*n1*n2/(d1*d2*d3);  
         phase = -phase; 
         sum = sum+norm*term; 
       };
      
     return sum;
  };


/* 
   this function tries to convert a decimal number
   into a "reasonable" fraction; 7/14157 not! ... 
*/ 
frac Dec2Frac(double N)
  {  
    int x,y,i,j; /* x & y are temp variable, and i,j are indices */
    int found;       /* a boolean variable */  
    frac num;     /* variable of type frac */
    double CG=N;

    if (CG < 0.0) { num.s ='-' ;} else { num.s='+' ;};
    CG = (CG*CG); 
    x = round(CG * 1E6);
    i = 1; j = 1; found = 0;
    if (x == 0) found = 1;
    while (i <= 100 && !found) 
         {
          j=i; 
      while(j <= 100 && !found)  
           {
            y = round((i*1.0/j*1.0)*1E6); 
                if (x == y) found = 1;
        j++; 
           }; 
      i++;
         };
        i--; j--;

    if (!found) i=-1; /* did not find a reasonable fraction */
    num.numer=i; num.denom=j;
    return num;
        
  };


/* VCo() determines the Clebsch-Gordan coefficient using the Wigner 3j coefficient */

void VCo(double j1,
         double j2,
         double j12,
         double m12)
  {
   
   double m1, m2, c; 
   char sign='=';
   frac f;  
 
   printf("|%1.1lf,%1.1lf;%1.1lf,%1.1lf>\n", j1, j2, j12, m12);
   for(m1=j1; m1>=-j1; m1--)
     {
       for(m2=j2; m2>=-j2; m2--)
        {
          if ((m1+m2)== m12)
            {
              c = W3j(j1,j2,j12,m1,m2,-m12); 
              c = pow(-1.0,(-j1+j2-m12))*pow((2*j12 + 1.0),0.5)*c;
                                                               
              f = Dec2Frac(c);

    		  if (f.numer == -1)
                {
                 printf("                    %c(%lf)",sign,c);
                }
              else
               {   
                if (f.numer==0 || f.numer==f.denom)
                 { 
		          printf("                    %c[  %i ]", sign, f.numer);
                 }
			    else
                 {
                  printf("                    %c[%c%i/%i]", sign, f.s, f.numer,f.denom);
	             };
		       };
               printf("|%1.1lf,%1.1lf;%1.1lf,%1.1lf> \n", j1, j2, m1, m2);  
			   sign='+';
            };
         };
      };
  };


int main(void)
  { 
    
    int c=0;
    double j1,j2,j12,m12;
	    
    

    printf("Addition of Two Angular Momenta j1 and j2\n");
	
            
    printf("\n\nInput j1 and j2 as decimals separated by commas or -1 to end: ");
    scanf("%lf,%lf",&j1,&j2);

    if ((j1 < 0) || (j2 < 0)) exit (0);     
   
    printf("\n     There are %1.0lf |j1,j2;j12,m12> states where,\n", (2*j1 + 1)*(2*j2 + 1));
    printf("\n |j1,j2;j12,m12> = \xE4 <j1,j2;m1,m2|j1,j2;j12,m12>|j1,j2;m1,m2>\n ");
	printf("\n              Note:    [\xF1\x61] \xF0 \xF1\xFB\x61 \n");

    for(j12=(j1+j2); j12>=fabs(j1-j2); j12--)   
      {
        for(m12=j12; m12>=-j12; m12--)      
          {
            c++; printf("\n%i. ", c);
            VCo(j1, j2, j12, m12); 
          };
        
      };
    printf("\n Done! Press any key...");
    while(!_getch() );
    printf("\n"); 
  };


