/*  
     The W3j procedure was derived from the C program described by William J. Thompson.
          William J. Thompson, Angular Momentum: An Illustrated Guide 
               to Rotational Symmetries for Physical Systems,
                    John Wiley and Sons Inc., 1994

     This program adds four angular momemta j1, j2, j3 and j4

          coupling scheme: j1+j2=j12, j3+j4=j34, j12+j34=J
*/

#define _USE_MATH_DEFINES 
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <math.h>
#include "am.h"

typedef struct fraction
 {
  int numer;  /* numerator */
  int denom;  /* denominator */
  char s;     /* sign of the fraction */
 } frac;

 /* W3j() determines the Wigner 3j coefficient using Racah's formula */

double W3j(double a, double b,
           double c, double ma,
           double mb, double mc)
           
  {
    double n1, n2, n3, d1, d2, d3;
    double norm, sum, phase, term;
    int minmax[3], tkmin, tkmax, tk;
    
  /*  Normalization Factorials  */
  
     n1 = Fctrl((c+a-b))*Fctrl((c-a+b));
     n2 = Fctrl((a+b-c));
     n3 = Fctrl((c-mc))*Fctrl((c+mc));
     d1 = Fctrl((a+b+c)+1);
     d2 = Fctrl((a-ma))*Fctrl((a+ma));
     d3 = Fctrl((b-mb))*Fctrl((b+mb));
 	 norm = Power(-1.0,(a-b-mc))*sqrt(n1*n2*n3/(d1*d2*d3));
     
     minmax[1] = 0;
     minmax[2] = b-a-mc; 
     tkmin = Max(minmax,2); 
     minmax[1] = c-a+b;  
     minmax[2] = c-mc;   
     tkmax = Min(minmax,2);
     sum = 0;
     phase = Power(-1.0, (tkmin+b+mb));
      
     for(tk= tkmin; tk <= tkmax; tk++)
       {
         n1 = Fctrl((b+c+ma-tk));
         n2 = Fctrl((a-ma+tk));
         d1 = Fctrl(tk)*Fctrl((c-a+b-tk));  
         d2 = Fctrl((c-mc-tk));
         d3 = Fctrl((tk+a-b+mc));
         term = phase*n1*n2/(d1*d2*d3);  
         phase = -phase; 
         sum = sum+norm*term; 
       };
      
     return sum;
  };

/* 
   this function tries to convert a decimal number
   into a "reasonable" fraction; 7/14157 not! ... 
*/ 
frac Dec2Frac(double N)
  {  
    int x,y,i,j; /* x & y are temp variable, and i,j are indices */
    int found;       /* a boolean variable */  
    frac num;     /* variable of type frac */
    double CG=N;

    if (CG < 0.0) { num.s ='-' ;} else { num.s='+' ;};
    CG = (CG*CG); 
    x = round(CG * 1E6);
    i = 1; j = 1; found = 0;
    if (x == 0) found = 1;
    while (i <= 1000 && !found) 
         {
          j=i; 
      while(j <= 1000 && !found)  
           {
            y = round((i*1.0/j*1.0)*1E6); 
                if (x == y) found = 1;
        j++; 
           }; 
      i++;
         };
        i--; j--;

    if (!found) i=-1; /* did not find a reasonable fraction */
    num.numer=i; num.denom=j;
    return num;
        
  };


/* VCo() determines the Clebsch-Gordan coefficient using the Wigner 3j coefficient */

void VCo(double j1,
         double j2,
	     double j3,
         double j4,
         double j12,  
         double j34,
	     double J,  
         double M)
  {
   
   double m1, m2, m3, m4, m12, m34, c1, c2, c3, c; 
   char sign='=';
   frac f;
 
   printf("|%1.1lf,%1.1lf,%1.1lf,%1.1f;%1.1lf,%1.1lf,%1.1lf,%1.1lf>\n", j1, j2, j3, j4, j12, j34, J, M);
   for(m12=-j12; m12<=j12; m12++)
     {
       for(m34=-j34; m34<=j34; m34++)
         {
	       for(m1=-j1; m1<=j1; m1++)
			 {
			   for(m2=-j2; m2<=j2; m2++)
			     {
				   for(m3=-j3; m3<=j3; m3++) 
                     {
					   for(m4=-j4; m4<=j4; m4++)
						 {
                           if ((m1+m2)==m12 && (m4+m3)==m34 && (m12+m34)==M) 
                             {
                              c1 = W3j(j1,j2,j12,m1,m2,-m12); 
                              c1 = pow(-1.0,(-j1+j2-m12))*pow((2*j12 + 1.0),0.5)*c1;
					          c2 = W3j(j3,j4,j34,m3,m4,-m34);
                              c2 = pow(-1.0,(-j3+j4-m34))*pow((2*j34 + 1.0),0.5)*c2;
                              c3 = W3j(j12,j34,J,m12,m34,-M);
                              c3 = pow(-1.0,(-j12+j34-M))*pow((2*J + 1.0),0.5)*c3;
                              c = c1*c2*c3; 	 
                              
                              f = Dec2Frac(c);

                              if (f.numer == -1)  /* could not get a fraction */
                                {
                                 printf("                    %c(%lf)",sign,c);
								}
                              else
                                {     
                                 if (f.numer==0 || f.numer==f.denom)
                                  { 
		                           printf("                    %c[  %i ]", sign, f.numer);
                                  }
			                     else
                                  {
                                   printf("                    %c[%c%i/%i]", sign, f.s, f.numer,f.denom);
	                              };
							    }
                               printf("|%1.1lf,%1.1lf,%1.1lf,%1.1lf;%1.1lf,%1.1lf,%1.1lf,%1.1lf> \n", j1, j2, j3, j4, m1, m2, m3, m4);
                               sign='+';
                                      
                             };
                         };
                     };
                 };
	         };
         };
	 };
	
  };


int main(void)
  { 
    
    int c=0; 
    double j1,j2,j3,j4,j12,j34,J,M;
    double states;
    

    printf("Addition of Four Angular Momenta j1, j2, j3 and j4\n");
	printf("\n coupling: j1+j2=j12,j3+j4=j34, j12+j34=J \n");
            
    printf("\n\nInput j1, j2, j3, and j4 as decimals separated by commas or -1 to end.\n");
	printf("\nEnter j1,j2,j3,j4: ");
    scanf("%lf,%lf,%lf,%lf",&j1,&j2,&j3,&j4);

    if ((j1 < 0) || (j2 < 0) || (j3 < 0) || (j4 < 0)) exit (0);     
    
	states = (2*j1 + 1)*(2*j2 + 1)*(2*j3 + 1)*(2*j4 + 1); 
    printf("\n     There are %1.0lf |j1,j2,j3,j4;j12,j34,J,M> states.\n",states );
    printf("\n              Note:    [\xF1\x61] \xF0 \xF1\xFB\x61 \n");

    
	for(j12=(j1+j2); j12>=fabs(j1-j2); j12--)
	  {
		for(j34=(j3+j4); j34>=(fabs(j3-j4)); j34--)
          {
			for(J=(j12+j34); J>=(fabs(j34-j12)); J--)
			  {
                for(M = J; M>=-J; M--)  
                  {
                   c++; printf("\n%i. ", c);
                   VCo(j1,j2,j3,j4,j12,j34,J,M);  
			      };
	          };
	      };
	  };
	printf("\n Done! Press any key...");
    while(!_getch() );
    printf("\n");  
  };


