/* This program is derived from the C program described by William J. Thompson.
   William J. Thompson, Angular Momentum: an illustrated guide 
          to rotational symmetries for physical systems,
                 John Wiley and Sons Inc., 1994.
*/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "am.h"

typedef struct fraction
 {
  int numer;  /* numerator */
  int denom;  /* denominator */
  char s;     /* sign of the fraction */
 } frac;


  /* Square of the delta factor */
  
double Delt(int ta, int tb, int tc)
  {
   
   return ( Fctrl((ta+tb-tc)/2)*
            Fctrl((ta+tc-tb)/2)*
            Fctrl((tb+tc-ta)/2) )/
            Fctrl((ta+tb+tc)/2+1);
   
  
  }
  
  
double _6j(int ta, int tb, int tc,
           int td, int te, int tf)
  {               
   double norm,sum,phase,term;
   double n1,d1,d2,d3,d4,d5,d6;
   int minray[4],maxray[6];
   int tkmin,tkmax,tk;
   
  /* Normalization factorials */
   
   norm = sqrt(Delt(ta,tb,te)*Delt(ta,tc,tf)*
               Delt(tb,td,tf)*Delt(tc,td,te)); 
               
  /* Minimum summation index */
    
   minray[1] = 0;
   minray[2] = ta+td-te-tf;
   minray[3] = tb+tc-te-tf;
   tkmin = Max(minray,3);
  
  /* Maximun summation index */
  
   maxray[1] = ta+tb+tc+td+2;
   maxray[2] = ta+tb-te;
   maxray[3] = tc+td-te;
   maxray[4] = ta+tc-tf;
   maxray[5] = tb+td-tf;
   tkmax = Min(maxray,5); 
  
  /* Phase of 6j */  
  
   phase = Power(-1.0,(ta+tb+tc+td+tkmin)/2);
  
   sum = 0;
   for (tk = tkmin; tk <= tkmax; tk = tk + 2)
     {
       n1 = Fctrl((ta+tb+tc+td-tk)/2+1);
       d1 = Fctrl(tk/2)*Fctrl((te+tf-ta-td+tk)/2);
       d2 = Fctrl((te+tf-tb-tc+tk)/2);
       d3 = Fctrl((ta+tb-te-tk)/2);
       d4 = Fctrl((tc+td-te-tk)/2);
       d5 = Fctrl((ta+tc-tf-tk)/2);
       d6 = Fctrl((tb+td-tf-tk)/2);
       term = phase*n1/(d1*d2*d3*d4*d5*d6);
       phase = -phase;
       sum = sum+norm*term;
     };
   return sum;
   
  }

double _9j(int ta, int tb, int tc,
	       int td, int te, int tf,
		   int tg, int th, int ti)
  {
   double sum,term;
   int minmax[4],tkmin,tkmax,tk;

   /* Minimum summation index */

   minmax[1] = abs(ta-ti);
   minmax[2] = abs(th-td);
   minmax[3] = abs(tb-tf);
   tkmin = Max(minmax,3);

  /* Maximun summation index */

   minmax[1] = ta+ti;
   minmax[2] = th+td;
   minmax[3] = tb+tf;
   tkmax = Min(minmax,3); 
   sum = 0;
   for (tk = tkmin; tk <= tkmax; tk = tk+2)
     {
      term = _6j(ta,ti,td,th,tk,tg)*
             _6j(tb,tf,th,td,tk,te)*
             _6j(ta,ti,tb,tf,tk,tc);
      sum = sum+(tk+1)*term; 
     }
   return sum;
  }  
 

/* 
   this function tries to convert a decimal number
   into a "reasonable" fraction; 7/14157 not! ... 
*/ 
frac Dec2Frac(double N)
  {  
    int x,y,i,j; /* x & y are temp variable, and i,j are indices */
    int found;       /* a boolean variable */  
    frac num;     /* variable of type frac */
    double CG=N;

    if (CG < 0.0) { num.s ='-' ;} else { num.s='+' ;};
    CG = (CG*CG); 
    x = round(CG * 1E6);
    i = 1; j = 1; found = 0;
    if (x == 0) found = 1;
    while (i <= 100 && !found) 
         {
          j=i; 
      while(j <= 100 && !found)  
           {
            y = round((i*1.0/j*1.0)*1E6); 
                if (x == y) found = 1;
        j++; 
           }; 
      i++;
         };
        i--; j--;

    if (!found) i=-1; /* did not find a reasonable fraction */
    num.numer=i; num.denom=j;
    return num;
        
  }
  
  
int main(void)
  {
   double Nine_j;
   double ja,jb,jc,jd,jab,jcd,jac,jbd,J;
   int ta,tb,tc,td,te,tf,tg,th,ti;
   frac _f; 
   
   printf("Angular momentum\n");
   printf("9j recoupling coefficients\n");
   printf("\n      W(ja,jb,jab,jc,jd,jcd,jac,jbd,J) \n");
   printf("\n          Note: [\xF1\x61] = \xF1\xFB\x61 \n");
   
   ta = 0;
   while(ta >= 0)
     {
      printf("\n\nInput ja as a decimals or -1 to end: ");
      scanf("%lf",&ja);
      ta = Twice(ja);
      if (ta < 0)
        {
         printf("\nEnd 9j recoupling coefficients\n");
         exit(0);
        }  
      printf("\nInput jb and jab separated by commas: ");
      scanf("%lf,%lf",&jb,&jab);
      printf("\nInput jc, jd and jcd separated by commas: ");
      scanf("%lf,%lf,%lf",&jc,&jd,&jcd);
      printf("\nInput jac, jbd and J separated by commas: ");
      scanf("%lf,%lf,%lf",&jac,&jbd,&J);

	  /*  Convert to inegers that are 2x input values */
      tb = Twice(jb);
      tc = Twice(jab);
      td = Twice(jc);
      te = Twice(jd);
      tf = Twice(jcd);
      tg = Twice(jac);
      th = Twice(jbd);
      ti = Twice(J);

      /* Testing for zero coefficient */
      
      if (TriangleBroken(ta,tb,tc) != 0 ||
          TriangleBroken(td,te,tf) != 0 ||
          TriangleBroken(tg,th,ti) != 0 ||
          TriangleBroken(ta,td,tg) != 0 ||
		  TriangleBroken(tb,te,th) != 0 ||
		  TriangleBroken(tc,tf,ti) != 0 ) printf( " try again");
      else
        {
         Nine_j = _9j(ta,tb,tc,td,te,tf,tg,th,ti);

         _f = Dec2Frac(Nine_j);

         if (_f.numer == -1) /* could not get a fraction */
           {
            printf("\n 6j coefficeint = %lf",Nine_j);
           }
         else
           {   
            if (_f.numer==0 || _f.numer==_f.denom)
              { 
                printf("\n 9j coefficeint =  %i  ", _f.numer);
              }
            else
              {
                printf("\n 9j coefficeint = [%c%i/%i] = %lf", _f.s, _f.numer,_f.denom, Nine_j);
              };
           };
         
        };
     };
     
   return 0;
   
  }     
                  
           
 

